
# q-expansion of newform 64.13.c.b, downloaded from the LMFDB on 04 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 64
weight = 13
poly_data = [289, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-24, 48], 1]]

hecke_ring_character_values = [[63, [-1, 0]], [5, [1, 0]]]
aps_data = [[0, 0], [0, -1], [270, 0], [0, 222], [0, 2025], [4386350, 0], [-39383550, 0], [0, -66525], [0, -294678], [-163560978, 0], [0, 283800], [-3600024050, 0], [2124864738, 0], [0, -3193359], [0, 21555396], [13585251150, 0], [0, -30494475], [-35496554258, 0], [0, -149495985], [0, -152996850], [-5982269150, 0], [0, 149070300], [0, -487109133], [-753989286942, 0], [-970603845950, 0], [-1007533838322, 0], [0, 1661670], [0, 392717889], [-1234863358162, 0], [3022162762050, 0], [0, 1122288096], [0, -9101793825], [4330437674850, 0], [0, 9534945225], [-7127536703922, 0], [0, 10144831050], [5931387357550, 0], [0, -36420494937], [0, 16254954774], [-27568580346450, 0], [0, 47330347275], [27436659255758, 0], [0, 3023600400], [-75123651356350, 0], [-88366149572850, 0], [0, -60914130450], [0, 156969021075], [0, 106000269096], [0, -7216936137], [-90640539380338, 0], [-35828168320350, 0], [0, -350804544300], [-53380422315902, 0], [0, -235907581275], [157921599643650, 0], [0, -122103899874], [538110704408238, 0], [0, 285917899500], [-576831774022450, 0], [30277422824418, 0], [0, 1055943465645], [210405331831950, 0], [0, -928687782933], [0, -1708994145150], [-880830260230750, 0], [1750615955952750, 0], [0, -350246072775], [1862989092333250, 0], [0, -656247465603], [-428497111285778, 0], [-40342795098750, 0], [0, 2502346198950], [0, -3854029727052], [-2602862437315250, 0], [0, 2636602189125], [0, 6431370287904], [3401368030528398, 0], [-510617279047250, 0], [-8144727841278, 0], [-6525612755269598, 0], [0, 1073494455975], [720279955598222, 0], [0, 3966513113700], [1884184697086850, 0], [0, -7605662507550], [0, -7714159555563], [6729412471777602, 0], [10998701333361250, 0], [1493459514173358, 0], [0, 2644041930492], [0, 4344351193107], [0, 10984202202600], [0, 15302980238214], [0, 12980427926625], [0, 11306373997275], [0, -30930232953582], [-827357175993618, 0], [30303347937658722, 0], [0, 4111566600681], [-34971438774108562, 0], [0, -38352872530233], [-23944370052532050, 0], [0, 28807841833515], [-25833671575038558, 0], [0, 3812025052725], [6043648901557250, 0], [0, -22265363790855], [-34467082353215550, 0], [0, 79111203658650], [19924423215899618, 0], [0, -95919099099960], [-3309054115502450, 0], [80432248130258850, 0], [0, -109200960554175], [0, 23729796174450], [-9072402051376062, 0], [0, 18039118191327], [0, -14102308005570], [102325252020619950, 0], [0, -166710173824125], [1027022344033102, 0], [-93596255416613950, 0], [123322431238836750, 0], [0, 46931952339729], [0, 78182757998475], [151419995047537902, 0], [-173159682754058098, 0], [0, -25320690533700], [0, -2670984549126], [-140075489281023250, 0], [0, 112032861480375], [0, -146162726172090], [0, 403922741942100], [72409556532659150, 0], [88097930747067042, 0], [-219640417324714942, 0], [-336478740090433650, 0], [0, -99456819075549], [421727225746474350, 0], [189059843995753698, 0], [0, 350661936003825], [-269622497538133938, 0], [0, 657077600892930], [0, 381022602588405], [-499181100363925522, 0], [0, 626092836223950], [62656044968870350, 0], [564927305591657250, 0], [0, -421803951036675], [0, -601071134944248], [561799308601135150, 0], [327518417184663618, 0], [0, 713482767333627], [0, 53511474529458], [0, -571849795228407], [0, -16141790978100], [0, 1310680407443850], [694106475825664962, 0], [-534685812568467550, 0], [-845209657030947282, 0], [0, -303051016000629], [-288377121698617950, 0], [0, 708651826558254], [0, -1182160025526375], [-1547583097352068350, 0], [0, -882968209898310], [0, 704092002733800], [-1644287449509931250, 0]]
